@@ -4,6 +4,7 @@
   Copyright (C) 1999-2000 by Matthias Kiefer
                             <matthias.kiefer@gmx.de>
                 2001-2004 by Stanislav Visnovsky <visnovsky@kde.org>
+  Copyright (C) 2005, 2006 by Nicolas GOUTTE <goutte@kde.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
   In addition, as a special exception, the copyright holders give
   permission to link the code of this program with any edition of
@@ -50,6 +51,7 @@
 #include "projectsettings.h"
 #include "findoptions.h"
 #include "cvsresources.h"
+#include "svnresources.h"
 
 class CatManListItem;
 class QPixmap;
@@ -62,6 +64,7 @@ class KConfig;
 class KDataToolInfo;
 class ValidateProgressDialog;
 class CVSHandler;
+class SVNHandler;
 class MarkPatternDialog;
 class ValidationOptions;
 
@@ -101,7 +104,8 @@ public:
    void setDirCommandsMenu( QPopupMenu *m);
    void setFileCommandsMenu( QPopupMenu *m);
 
-   CVSHandler * cvsHandler( );
+   CVSHandler * cvsHandler();
+   SVNHandler * svnHandler();
 
 public slots:
    void setSettings(KBabel::CatManSettings newSettings);
@@ -164,6 +168,22 @@ public slots:
    void cvsCommitMarkedTemplate( );
    void cvsDiff( );
 
+   void svnUpdate( );
+   void svnUpdateMarked( );
+   void svnCommit( );
+   void svnCommitMarked( );
+   void svnStatusRemote();
+   void svnStatusRemoteMarked();
+   void svnStatusLocal();
+   void svnStatusLocalMarked();
+   void svnUpdateTemplate( );
+   void svnUpdateMarkedTemplate( );
+   void svnCommitTemplate( );
+   void svnCommitMarkedTemplate( );
+   void svnDiff( );
+   void svnInfo();
+   void svnInfoMarked();
+
    QString find(KBabel::FindOptions &options, QStringList &rest);
 
    void showLog();
@@ -207,8 +227,8 @@ public slots:
 
    /**
     * An update for more than one file has become necessary. For instance
-    * after 'cvs commit' the file contents have not changed but the cvs file
-    * status could have changed.
+    * after 'cvs commit' or 'svn commit' the file contents have not changed
+    * but the CVS/SVN file status could have changed.
     */
    void updateFiles( const QStringList& files );
 
@@ -348,10 +368,12 @@ private slots:
    void processEnded(KProcess *proc);
    void columnClicked(QListViewItem * item, const QPoint & pnt, int c);
 
-   void slotToggleCVSColumn( bool );
+   void slotToggleCVSOrSVNColumn( bool );
 
-   void slotValidPORepository( bool );
-   void slotValidPOTRepository( bool );
+   void slotValidPOCVSRepository( bool );
+   void slotValidPOSVNRepository( bool );
+   void slotValidPOTCVSRepository( bool );
+   void slotValidPOTSVNRepository( bool );
 
 private:
    void toggleColumn( uint id, bool show);
@@ -384,6 +406,7 @@ private:
    void validate_internal( const QStringList&, const KDataToolInfo &, const QString& );
 
    void doCVSCommand( CVS::Command cmd, bool marked = false, bool templates = false );
+   void doSVNCommand( SVN::Command cmd, bool marked = false, bool templates = false );
 
 private:
    QDict<CatManListItem> _dirList;
@@ -424,9 +447,17 @@ private:
 
    KBabel::KBabelMailer* mailer;
 
-   CVSHandler * cvshandler;
-   bool validPORepository;
-   bool validPOTRepository;
+   CVSHandler* cvshandler;
+   SVNHandler* svnhandler;
+
+   /// Is the PO path a valid CVS repository?
+   bool m_validPOCVSRepository;
+   /// Is the POT path a valid CVS repository?
+   bool m_validPOTCVSRepository;
+   /// Is the PO path a valid SVN repository?
+   bool m_validPOSVNRepository;
+   /// Is the POT path a valid SVN repository?
+   bool m_validPOTSVNRepository;
 
    MarkPatternDialog * markPatternDialog;
 
