@@ -148,7 +148,7 @@ SVNHandler::FileStatus SVNHandler::fstatus( const QString& filename ) const
   QDomElement entry, wcStatus;
 
   // Parse the output.
-  if ( !doc.setContent( out.output(), &errorMsg, &errorLine, &errorCol ) ) {
+  if ( !doc.setContent( out.getOutput(), &errorMsg, &errorLine, &errorCol ) ) {
     kdDebug(8109) << "Cannot parse \"svn status -v --xml\" output for"
         << filename << endl << "Line: " << errorLine << " Column: "
         << errorCol << " Error: " << errorMsg << endl;
