@@ -3,10 +3,9 @@
 /* ****************************************************************************
   This file is part of KBabel
 
-  Copyright (C) 1999-2000 by Matthias Kiefer
-                            <matthias.kiefer@gmx.de>
-		2001-2003 by Stanislav Visnovsky
-			    <visnovsky@kde.org>
+  Copyright (C) 1999-2000 by Matthias Kiefer <matthias.kiefer@gmx.de>
+		2001-2003 by Stanislav Visnovsky <visnovsky@kde.org>
+  Copyright (C) 2006 by Nicolas GOUTTE <nicolasg@snafu.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -50,7 +49,7 @@
 #include <kgenericfactory.h>
 #include <klocale.h>
 
-K_EXPORT_COMPONENT_FACTORY( kbabel_gettextimport, KGenericFactory<GettextImportPlugin> )
+K_EXPORT_COMPONENT_FACTORY( kbabel_gettextimport, KGenericFactory<GettextImportPlugin> ( "kbabelgettextimportfilter" ) )
 
 using namespace KBabel;
 
@@ -60,6 +59,8 @@ GettextImportPlugin::GettextImportPlugin(QObject* parent, const char* name, cons
 
 ConversionStatus GettextImportPlugin::load(const QString& filename, const QString&)
 {
+   kdDebug( KBABEL ) << k_funcinfo << endl;
+   
    if ( filename.isEmpty() ) {
       kdDebug(KBABEL) << "fatal error: empty filename to open" << endl;
       return NO_FILE;
@@ -75,10 +76,6 @@ ConversionStatus GettextImportPlugin::load(const QString& filename, const QStrin
 
    QFile file(filename);
 
-
-   bool recoveredError=false;
-   bool docbookFile=false;
-   	
    if ( !file.open( IO_ReadOnly ) )
       return NO_PERMISSIONS;
    
@@ -92,9 +89,23 @@ ConversionStatus GettextImportPlugin::load(const QString& filename, const QStrin
    bool hadCodec;
    QTextCodec* codec=codecForArray( ba, &hadCodec );
    
+   bool recoveredErrorInHeader = false;
+   
    QTextStream stream(ba,IO_ReadOnly);
+
    if(codec)
       stream.setCodec(codec);
+   else
+   {
+      kdWarning() << "No encoding declared or found, using UTF-8" << endl;
+      stream.setEncoding( QTextStream::UnicodeUTF8 );
+#ifdef __GNUC__
+# warning Default UTF-8 encoding needs to be improved
+#endif
+      // Templates define CHARSET, so if e make it a recoverable error, the template is not loadable anymore, as for templates recoverable errors are disqualifying.
+      //recoveredErrorInHeader = true;
+   }
+   
    QIODevice *dev = stream.device();
    int fileSize = dev->size();
 
@@ -106,20 +117,46 @@ ConversionStatus GettextImportPlugin::load(const QString& filename, const QStrin
    kdDebug(KBABEL) << "start parsing..." << endl;
 
    // first read header
-   ConversionStatus status = readHeader(stream);
+   const ConversionStatus status = readHeader(stream);
 
-   if(status!=OK)
+
+   if ( status == RECOVERED_PARSE_ERROR )
+   {
+      kdDebug( KBABEL ) << "Recovered error in header entry" << endl;
+      recoveredErrorInHeader = true;
+   }
+   else if ( status != OK )
    {
       emit signalClearProgressBar();
 
-      if( status == RECOVERED_PARSE_ERROR ) status = HEADER_ERROR;
-
+      kdDebug( KBABEL ) << "Parse error in header entry" << endl;
       return status;
    }
-
-   tempHeader.setMsgid( _msgid );
-   tempHeader.setMsgstr( _msgstr );
-   tempHeader.setComment( _comment );
+   
+   kdDebug() << "HEADER MSGID: " << _msgid << endl;
+   kdDebug() << "HEADER MSGSTR: " << _msgstr << endl;
+   if ( !_msgid.isEmpty() && !_msgid.first().isEmpty() )
+   {
+      // The header must have an empty msgid
+      kdWarning(KBABEL) << "Header entry has non-empty msgid. Creating a temporary header! " << _msgid << endl;
+      tempHeader.setMsgid( QStringList() );
+      QStringList tmp;
+      tmp.append(
+         "Content-Type: text/plain; charset=UTF-8\\n" // Unknown charset
+         "Content-Transfer-Encoding: 8bit\\n"
+         "Mime-Version: 1.0" );
+      tempHeader.setMsgstr( tmp );
+      // We keep the comment of the first entry, as it might really be a header comment (at least partially)
+      const QString comment( "# Header entry was created by KBabel!\n#\n" + _comment );
+      tempHeader.setComment( comment );
+      recoveredErrorInHeader = true;
+   }
+   else
+   {
+      tempHeader.setMsgid( _msgid );
+      tempHeader.setMsgstr( _msgstr );
+      tempHeader.setComment( _comment );
+   }
    if(tempHeader.isFuzzy())
    {
       tempHeader.removeFuzzy();
@@ -129,11 +166,12 @@ ConversionStatus GettextImportPlugin::load(const QString& filename, const QStrin
    const bool docbookContent = (tempHeader.msgstr().find("application/x-xml2pot") != tempHeader.msgstr().end());
 
    // now parse the rest of the file
-   ConversionStatus success=OK;
    uint counter=0;
    QValueList<uint> errorIndex;
+   bool recoveredError=false;
+   bool docbookFile=false;
 
-   while(!stream.eof() && success==OK )
+   while( !stream.eof() )
    {
       kapp->processEvents(10);
       if( isStopped() )
@@ -141,13 +179,12 @@ ConversionStatus GettextImportPlugin::load(const QString& filename, const QStrin
          return STOPPED;
       }
 
-      success=readEntry(stream);
+      const ConversionStatus success=readEntry(stream);
 
       if(success==OK)
       {
          if( _obsolete )
          {
-               success=OK;
                tempObsolete.append(_comment);
          }
          else
@@ -168,7 +205,7 @@ ConversionStatus GettextImportPlugin::load(const QString& filename, const QStrin
       }
       else if(success==RECOVERED_PARSE_ERROR)
       {
-         success=OK;
+         kdDebug( KBABEL ) << "Recovered parse error in entry: " << counter << endl;
          recoveredError=true;
          errorIndex.append(counter);
          
@@ -182,6 +219,16 @@ ConversionStatus GettextImportPlugin::load(const QString& filename, const QStrin
          // add new entry to the list of entries
          appendCatalogItem(tempCatItem);
       }
+      else if ( success == PARSE_ERROR )
+      {
+         kdDebug( KBABEL ) << "Parse error in entry: " << counter << endl;
+         return PARSE_ERROR;
+      }
+      else
+      {
+         kdWarning( KBABEL ) << "Unknown success status, assumig parse error " << success << endl;
+         return PARSE_ERROR;
+      }
       counter++;
 
       const uint newPercent = (100*dev->at())/fileSize;
@@ -199,38 +246,47 @@ ConversionStatus GettextImportPlugin::load(const QString& filename, const QStrin
 
    emit signalClearProgressBar();
 
-   kdDebug(KBABEL) << "ready." << endl;
-      
-      if(counter==0)
-      {
-        // no items read
-	return PARSE_ERROR;
-      }
 
-   if(success!=PARSE_ERROR)
-   {
-      setGeneratedFromDocbook(docbookContent || docbookFile);
-
-      setHeader(tempHeader);
-      setCatalogExtraData(tempObsolete);
-      setErrorIndex(errorIndex);
-
-      if(hadCodec)
-         setFileCodec(codec);
-      else
-         setFileCodec(0);
-   }
-   else
+   // ### TODO: can we check that there is no useful entry?
+   if ( !counter )
    {
+      // Empty file? (Otherwise, there would be a try of getting an entry and the count would be 1 !)
+      kdDebug( KBABEL ) << k_funcinfo << " Empty file?" << endl;
       return PARSE_ERROR;
    }
+
+   kdDebug(KBABEL) << k_funcinfo << " ready" << endl;
+
+   // We have succesfully loaded the file (perhaps with recovered errors)
+   
+   setGeneratedFromDocbook(docbookContent || docbookFile);
+
+   setHeader(tempHeader);
+   setCatalogExtraData(tempObsolete);
+   setErrorIndex(errorIndex);
+
+   if(hadCodec)
+      setFileCodec(codec);
+   else
+      setFileCodec(0);
    
    setMimeTypes( "application/x-gettext" );
 
-   if(recoveredError)
-   	return RECOVERED_PARSE_ERROR;
-
-   return OK;
+   if ( recoveredErrorInHeader )
+   {
+      kdDebug( KBABEL ) << k_funcinfo << " Returning: header error" << endl;
+      return RECOVERED_HEADER_ERROR;
+   }
+   else if ( recoveredError )
+   {
+      kdDebug( KBABEL ) << k_funcinfo << " Returning: recovered parse error" << endl;
+      return RECOVERED_PARSE_ERROR;
+   }
+   else
+   {
+      kdDebug( KBABEL ) << k_funcinfo << " Returning: OK! :-)" << endl;
+      return OK;
+   }
 }
 
 QTextCodec* GettextImportPlugin::codecForArray(QByteArray& array, bool* hadCodec)
@@ -241,9 +297,9 @@ QTextCodec* GettextImportPlugin::codecForArray(QByteArray& array, bool* hadCodec
    }
 
    QTextStream stream( array, IO_ReadOnly );
-   // If needed this will detect a UTF-16 BOM (Byte Order mark), otherwise we assume that the header is in ASCII.
-   stream.setEncoding( QTextStream::UnicodeUTF8 );
-   // ### Qt4: set encoding to UTF-8 and set Unicode auto-detection
+   // ### TODO Qt4: see if it can be done with QByteArray alone, in an encoding-neutral way.
+   // Set ISO-8859-1 as it is a relatively neutral encoding when reading (compared to UTF-8  or a random locale encoing)
+   stream.setEncoding( QTextStream::Latin1 );
 
    // first read header
    ConversionStatus status = readHeader(stream);
@@ -292,14 +348,12 @@ QTextCodec* GettextImportPlugin::codecForArray(QByteArray& array, bool* hadCodec
       if(!codec)
       {
          kdWarning(KBABEL) << "charset found, but no codec available, using UTF-8 instead" << endl;
-	 codec=QTextCodec::codecForName("utf8");
       }
    }
    else
    {
       // No charset? So it is probably ASCII, therefore UTF-8
       kdWarning(KBABEL) << "No charset defined! Assuming UTF-8!" << endl;
-      codec=QTextCodec::codecForName("utf8");
    }
          
 
@@ -328,10 +382,10 @@ ConversionStatus GettextImportPlugin::readHeader(QTextStream& stream)
 
 ConversionStatus GettextImportPlugin::readEntry(QTextStream& stream)
 {
+   //kdDebug( KBABEL ) << k_funcinfo << " START" << endl;
    enum {Begin,Comment,Msgctxt,Msgid,Msgstr} part=Begin;
 
    QString line;
-   bool cancelLoop=false;
    bool error=false;
    bool recoverableError=false;
    bool seenMsgctxt=false;
@@ -346,19 +400,21 @@ ConversionStatus GettextImportPlugin::readEntry(QTextStream& stream)
    
    QStringList::Iterator msgstrIt=_msgstr.begin();
    
-   while(!stream.eof() && !cancelLoop)
+   while( !stream.eof() )
    {
        const int pos=stream.device()->at();
 
        line=stream.readLine();
 
+       //kdDebug() << "Parsing line: " << line << endl;
+
        // ### Qt4: no need of a such a check
        if(line.isNull()) // file end
           break;
        else if ( line.startsWith( "<<<<<<<" ) || line.startsWith( "=======" ) || line.startsWith( ">>>>>>>" ) )
        {
           // We have found a CVS/SVN conflict marker. Abort.
-          // (It cannot be any useful data of the PO file, as otheriwse the line would start with a quote)
+          // (It cannot be any useful data of the PO file, as otherwise the line would start with at least a quote)
           kdError(KBABEL) << "CVS/SVN conflict marker found! Aborting!" << endl << line << endl;
           return PARSE_ERROR;
        }
@@ -419,10 +475,9 @@ ConversionStatus GettextImportPlugin::readEntry(QTextStream& stream)
            }
            else
            {
-              kdDebug(KBABEL) << "no comment or msgid found after a comment: " << line << endl;
-
-               cancelLoop=true;
+               kdDebug(KBABEL) << "no comment, msgctxt or msgid found after a comment: " << line << endl;
                error=true;
+               break;
            }
        }
        else if(part==Comment)
@@ -476,9 +531,8 @@ ConversionStatus GettextImportPlugin::readEntry(QTextStream& stream)
             else
             {
                kdDebug(KBABEL) << "no comment or msgid found after a comment while parsing: " << _comment << endl;
-
                error=true;
-               cancelLoop=true;
+               break;
             }
         }
         else if(part==Msgctxt)
@@ -524,9 +578,8 @@ ConversionStatus GettextImportPlugin::readEntry(QTextStream& stream)
             else
             {
                kdDebug(KBABEL) << "no msgid found after a msgctxt while parsing: " << _msgctxt << endl;
-
                error=true;
-               cancelLoop=true;
+               break;
             }
         }
         else if(part==Msgid)
@@ -627,13 +680,18 @@ ConversionStatus GettextImportPlugin::readEntry(QTextStream& stream)
                if(!line.isEmpty())
                   recoverableError=true;
             }
- 
-            else if(line.startsWith("#") || line.startsWith("msgid"))
+            else if ( line.startsWith( "#" ) )
             {
-               kdDebug(KBABEL) << "no msgstr or msgid found after a msgid while parsing: " << _msgid.first() << endl;
-
-               cancelLoop=true;
+               // ### TODO: could this be considered recoverable?
+               kdDebug(KBABEL) << "comment found after a msgid while parsing: " << _msgid.first() << endl;
                error=true;
+               break;
+            }
+            else if ( line.startsWith( "msgid" ) )
+            {
+               kdDebug(KBABEL) << "Another msgid found after a msgid while parsing: " << _msgid.first() << endl;
+               error=true;
+               break;
             }
             // a line of the msgid with a missing quotation mark
             else if(line.find(QRegExp("^\"?.+\\n?\"?$")) != -1)
@@ -658,10 +716,9 @@ ConversionStatus GettextImportPlugin::readEntry(QTextStream& stream)
             }
             else
             {
-               kdDebug(KBABEL) << "no msgstr or msgid found after a msgid while parsing: " << _msgid.first() << endl;
-
-               cancelLoop=true;
+               kdDebug(KBABEL) << "no msgstr found after a msgid while parsing: " << _msgid.first() << endl;
                error=true;
+               break;
             }
         }
         else if(part==Msgstr)
@@ -703,16 +760,15 @@ ConversionStatus GettextImportPlugin::readEntry(QTextStream& stream)
             }
             else if((line.find(QRegExp("^\\s*msgid")) != -1) || (line.find(QRegExp("^\\s*#")) != -1))
             {
-               cancelLoop=true;
+               // We have read successfully one entry, so end loop.
                stream.device()->at(pos);// reset position in stream to beginning of this line
                break;
             }
             else if(line.startsWith("msgstr"))
             {
-               kdDebug(KBABEL) << "no msgid or comment found after a msgstr while parsing: " << _msgstr.last() << endl;
-
-               cancelLoop=true;
+               kdDebug(KBABEL) << "Another msgstr found after a msgstr while parsing: " << _msgstr.last() << endl;
                error=true;
+               break;
             }
             // another line of the msgstr with a missing quotation mark
             else if(line.find(QRegExp("^\"?.+\\n?\"?$")) != -1)
@@ -731,9 +787,8 @@ ConversionStatus GettextImportPlugin::readEntry(QTextStream& stream)
             else
             {
                kdDebug(KBABEL) << "no msgid or comment found after a msgstr while parsing: " << _msgstr.last() << endl;
-
-               cancelLoop=true;
                error=true;
+               break;
             }
         }
     }
@@ -752,7 +807,9 @@ ConversionStatus GettextImportPlugin::readEntry(QTextStream& stream)
            counter++;
        }
    }
-  */ 
+  */
+
+    //kdDebug( KBABEL ) << k_funcinfo << " NEAR RETURN" << endl;
     if(error)
        return PARSE_ERROR;
 	else if(recoverableError)
