@@ -18,7 +18,7 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
   In addition, as a special exception, the copyright holders give
   permission to link the code of this program with any edition of
@@ -86,10 +86,10 @@ public:
     /**
      * Default Constructor
      */
-    KBabelMW(QString projectFile = KBABEL_DEFAULTPROJECT);
+    KBabelMW(QString projectFile = QString());
 
     /** use this contructor, if you just want to create a new view of an existing catalog*/
-    KBabelMW(KBCatalog* catalog, QString projectFile = KBABEL_DEFAULTPROJECT);
+    KBabelMW(KBCatalog* catalog, QString projectFile = QString());
 
     /**
      * Default Destructor
