@@ -65,6 +65,20 @@ public:
 	*/
    int beginOfMarkedText();
 
+    /** 
+    * @returns the position in text, where the last marked text began
+    * or the current cursor position if there was no marked text.
+    * This is used for getting the start position for a text replacement
+    */
+   int beginOfLastMarkedText();
+
+    /** 
+    * @returns the position in text, where the last marked text ended
+    * or the current cursor position if there was no marked text.
+    * This is used for getting the end position for a text replacement
+    */
+   int endOfLastMarkedText();
+
    virtual void insertAt ( const QString & s, int line, int col, bool mark = false );
    virtual void removeLine ( int line );
 
@@ -92,6 +106,7 @@ public:
    bool isOverwriteMode() { return _overwrite; }    
 
 public slots:
+   
    virtual void clear();
    virtual void paste();
    virtual void setReadOnly(bool on);
@@ -99,6 +114,8 @@ public slots:
    virtual void setText(const QString& s);
    virtual void doKeyboardAction( KeyboardAction action );
    virtual void removeSelectedText(int selNum = 0);
+
+   virtual void onSelectionChanged();
    
    /**
    reimplemented overwrite mode, since QTextEdit handles this internally and does
@@ -128,6 +145,11 @@ protected:
    uint _lastParagraph;
    uint _lastParagraphOffset;
 
+   /* We save the last selection positions. This is needed when a tag is inserted to get the
+      left cursor position of the originally used selection */
+   int _lastSelectionStart;
+   int _lastSelectionEnd;
+
    /* flag to skip any work on updating, since it will be more changes */
    bool _dontUpdate;
 
