@@ -3,6 +3,7 @@
 
 #include "person.h"
 #include "bugdetailspart.h"
+#include "bugdetailsimpl.h"
 
 #include <qvaluelist.h>
 
@@ -30,9 +31,14 @@ public:
     QString compiler() const;
     QString os() const;
     BugDetailsPart::List parts() const;
+    void addAttachmentDetails( const QValueList<BugDetailsImpl::AttachmentDetails>& attch );
+    QValueList<BugDetailsImpl::AttachmentDetails> attachmentDetails() const;
     QValueList<BugDetails::Attachment> extractAttachments() const;
     static QValueList<BugDetails::Attachment> extractAttachments( const QString& text );
 
+    QDateTime submissionDate() const;
+    int age() const;
+
     bool operator==( const BugDetails &rhs );
 
     bool isNull() const { return m_impl == 0; }
