@@ -43,7 +43,10 @@ public:
 
     void setBug( const Bug &, const BugDetails & );
 
+    CWBugDetails *bugDetailsWidget() { return m_bugDetails; }
+
 public slots:
+    void setNoBug();
     void setLoading( const Bug &bug );
     void setCacheMiss( const Bug &bug );
 
