@@ -2,7 +2,7 @@
     cwsearchwidget.h  -  Search Pane
     
     copyright   : (c) 2001 by Martijn Klingens
-    email       : mklingens@yahoo.com
+    email       : klingens@kde.org
 
     *************************************************************************
     *                                                                       *
