@@ -4,7 +4,7 @@
 #include <kdialogbase.h>
 
 class QComboBox;
-class QTextEdit;
+class KTextEdit;
 
 class MessageEditor : public KDialogBase {
     Q_OBJECT
@@ -24,7 +24,7 @@ class MessageEditor : public KDialogBase {
     
   private:
     QComboBox *mSelectionCombo;
-    QTextEdit *mMessageEdit;
+    KTextEdit *mMessageEdit;
     
     QString mCurrentButton;
     QMap <QString,QString> mMessageButtons;
