@@ -1,8 +1,3 @@
-/*
- *  copyright (C) 2003-2004
- *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
- */
-
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -10,6 +5,8 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   copyright (C) 2003-2006                                               *
+ *   Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                  *
  ***************************************************************************/
 
 #ifndef CLASSIFIERLISTITEM_H
@@ -46,7 +43,9 @@ public:
      * @param name              The name of the operation.
      * @param id                The id of the operation.
      */
-    UMLClassifierListItem(const UMLObject *parent, QString Name, Uml::IDType id = Uml::id_None);
+    UMLClassifierListItem(const UMLObject *parent,
+                          const QString& name,
+                          Uml::IDType id = Uml::id_None);
 
     /**
      * Constructor.  Empty.
@@ -108,7 +107,7 @@ public:
      * @param parent    The parent widget.
      * @return  True for success of this operation.
      */
-    virtual bool showPropertiesDialogue(QWidget* parent) = 0;
+    virtual bool showPropertiesDialog(QWidget* parent) = 0;
 
     /**
      * Copy the internal presentation of this object into the new
