@@ -5,8 +5,8 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *  copyright (C) 2004-2006                                                *
- *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>                  *
+ *   copyright (C) 2004-2007                                               *
+ *   Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                  *
  ***************************************************************************/
 
 // own header
@@ -70,8 +70,8 @@ void ClassifierWidget::init() {
     m_bShowAttributes = ops.classState.showAtts;
     m_bShowStereotype = ops.classState.showStereoType;
     m_bDrawAsCircle = false;
-    setShowAttSigs( ops.classState.showAttSig );
     m_pAssocWidget = NULL;
+    setShowAttSigs( ops.classState.showAttSig );
 }
 
 void ClassifierWidget::updateSigs() {
@@ -723,7 +723,7 @@ void ClassifierWidget::toggleDrawAsCircle() {
 
 void ClassifierWidget::changeToClass() {
     WidgetBase::setBaseType(Uml::wt_Class);
-    getClassifier()->setInterface(false);
+    getClassifier()->setBaseType(Uml::ot_Class);
 
     const Settings::OptionState& ops = m_pView->getOptionState();
     m_bShowAttributes = ops.classState.showAtts;
@@ -735,7 +735,7 @@ void ClassifierWidget::changeToClass() {
 
 void ClassifierWidget::changeToInterface() {
     WidgetBase::setBaseType(Uml::wt_Interface);
-    getClassifier()->setInterface(true);
+    getClassifier()->setBaseType(Uml::ot_Interface);
 
     m_bShowAttributes = false;
     m_bShowStereotype = true;
@@ -744,6 +744,16 @@ void ClassifierWidget::changeToInterface() {
     update();
 }
 
+void ClassifierWidget::adjustAssocs(int x, int y) {
+    UMLWidget::adjustAssocs(x, y);
+
+    if (m_pDoc->loading() || m_pAssocWidget == 0) {
+        return;
+    }
+
+    m_pAssocWidget->computeAssocClassLine();
+}
+
 void ClassifierWidget::saveToXMI(QDomDocument & qDoc, QDomElement & qElement) {
     QDomElement conceptElement;
     UMLClassifier *umlc = getClassifier();
