@@ -5,8 +5,8 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *  copyright (C) 2004-2006                                                *
- *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>                  *
+ *   copyright (C) 2004-2006                                               *
+ *   Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                  *
  ***************************************************************************/
 
 #ifndef CLASSIFIERWIDGET_H
@@ -269,6 +269,18 @@ public:
      */
     void draw(QPainter & p, int offsetX, int offsetY);
 
+    /**
+     * Extends base method to adjust also the association of a class
+     * association.
+     * Executes the base method and then, if file isn't loading and the
+     * classifier acts as a class association, the association position is
+     * updated.
+     *
+     * @param x The x-coordinate.
+     * @param y The y-coordinate.
+     */
+    virtual void adjustAssocs(int x, int y);
+
     /**
      * Creates the <classwidget> or <interfacewidget >XML element.
      */
