@@ -5,15 +5,19 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2002-2006                                               *
+ *   copyright (C) 2002-2007                                               *
  *   Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                  *
  ***************************************************************************/
 
+// own header
+#include "umlclipboard.h"
+
+// qt/kde includes
 #include <kdebug.h>
 #include <kmessagebox.h>
 #include <klocale.h>
 
-#include "umlclipboard.h"
+// local includes
 #include "umldrag.h"
 #include "idchangelog.h"
 #include "../associationwidget.h"
@@ -64,6 +68,7 @@ QMimeSource* UMLClipboard::copy(bool fromView/*=false*/) {
         }
         m_AssociationList = view->getSelectedAssocs();
         view->copyAsImage(png);
+
     } else { //if the copy action is being performed from the ListView
         if(!listView->getSelectedItems(selectedItems)) {
             return 0;
@@ -102,7 +107,7 @@ QMimeSource* UMLClipboard::copy(bool fromView/*=false*/) {
     int i =0;
     switch(m_type) {
     case clip1:
-        data = new UMLDrag(m_ObjectList, m_ItemList);
+        data = new UMLDrag(m_ObjectList);
         break;
     case clip2:
         data = new UMLDrag(m_ObjectList, m_ItemList, m_ViewList);
@@ -113,14 +118,14 @@ QMimeSource* UMLClipboard::copy(bool fromView/*=false*/) {
     case clip4:
         if(png) {
             UMLView *view = UMLApp::app()->getCurrentView();
-            data = new UMLDrag(m_ObjectList, m_ItemList, m_WidgetList,
+            data = new UMLDrag(m_ObjectList, m_WidgetList,
                                m_AssociationList, *png, view->getType());
         } else {
             return 0;
         }
         break;
     case clip5:
-        data = new UMLDrag(m_ObjectList, m_ItemList, i);
+        data = new UMLDrag(m_ObjectList, i);
         // The int i is used to differentiate
         // which UMLDrag constructor gets called.
         break;
@@ -333,11 +338,8 @@ void UMLClipboard::CleanAssociations(AssociationWidgetList& associations) {
 /** If clipboard has mime type application/x-uml-clip1,
 Pastes the data from the clipboard into the current Doc */
 bool UMLClipboard::pasteClip1(QMimeSource* data) {
-    UMLDoc *doc = UMLApp::app()->getDocument();
-    UMLListViewItemList itemdatalist;
     UMLObjectList objects;
-    IDChangeLog* idchanges = doc->getChangeLog();
-    if (! UMLDrag::decodeClip1(data, objects, itemdatalist)) {
+    if (! UMLDrag::decodeClip1(data, objects)) {
         return false;
     }
     UMLListView *lv = UMLApp::app()->getListView();
@@ -453,8 +455,6 @@ bool UMLClipboard::pasteClip3(QMimeSource* data) {
 Pastes the data from the clipboard into the current Doc */
 bool UMLClipboard::pasteClip4(QMimeSource* data) {
     UMLDoc *doc = UMLApp::app()->getDocument();
-    UMLListViewItemList itemdatalist;
-    itemdatalist.setAutoDelete(false);
 
     UMLObjectList objects;
     objects.setAutoDelete(false);
@@ -470,7 +470,7 @@ bool UMLClipboard::pasteClip4(QMimeSource* data) {
 
     Uml::Diagram_Type diagramType;
 
-    if( !UMLDrag::decodeClip4(data, objects, itemdatalist, widgets, assocs, diagramType) ) {
+    if( !UMLDrag::decodeClip4(data, objects, widgets, assocs, diagramType) ) {
         return false;
     }
 
@@ -486,15 +486,17 @@ bool UMLClipboard::pasteClip4(QMimeSource* data) {
     if(!idchanges) {
         return false;
     }
-    //make sure the file we are pasting into has the objects
-    //we need if there are widgets to be pasted
-    UMLObject* obj = 0;
-    while ( (obj=object_it.current()) != 0 ) {
-        ++object_it;
+     //make sure the file we are pasting into has the objects
+     //we need if there are widgets to be pasted
+     UMLObject* obj = 0;
+     while ( (obj=object_it.current()) != 0 ) {
+         ++object_it;
+
         if(!doc->assignNewIDs(obj)) {
             return false;
         }
-    }
+
+     }
 
     //now add any widget we are want to paste
     bool objectAlreadyExists = false;
@@ -505,11 +507,16 @@ bool UMLClipboard::pasteClip4(QMimeSource* data) {
     while ( (widget=widget_it.current()) != 0 ) {
         ++widget_it;
 
-        if (currentView->findWidget(idchanges->findNewID(widget->getID()))) {
+        Uml::IDType oldId = widget->getID();
+        Uml::IDType newId = idchanges->findNewID(oldId);
+        if (currentView->findWidget(newId)) {
+            kError() << "UMLClipboard::pasteClip4: widget (oldID=" << ID2STR(oldId)
+                << ", newID=" << ID2STR(newId) << ") already exists in target view."
+                << endl;
+            widgets.remove(widget);
+            delete widget;
             objectAlreadyExists = true;
-        }
-
-        if ( !currentView->addWidget(widget, true) ) {
+        } else if (! currentView->addWidget(widget, true)) {
             currentView->endPartialWidgetPaste();
             return false;
         }
@@ -530,6 +537,7 @@ bool UMLClipboard::pasteClip4(QMimeSource* data) {
     currentView->activate();
     currentView->endPartialWidgetPaste();
 
+    /*
     UMLListView *listView = UMLApp::app()->getListView();
     UMLListViewItem* item = 0;
     UMLListViewItem* itemdata = 0;
@@ -545,7 +553,7 @@ bool UMLClipboard::pasteClip4(QMimeSource* data) {
             }
         }
         ++it;
-    }
+        }*/
 
     if (objectAlreadyExists) {
         pasteItemAlreadyExists();
@@ -569,53 +577,62 @@ bool UMLClipboard::pasteClip5(QMimeSource* data) {
         << endl;
         return false;
     }
-    UMLListViewItemList itemdatalist;
+
     UMLObjectList objects;
     objects.setAutoDelete(false);
     IDChangeLog* idchanges = 0;
-    bool result = UMLDrag::decodeClip5(data, objects, itemdatalist, parent);
+    bool result = UMLDrag::decodeClip5(data, objects, parent);
 
     if(!result) {
         return false;
     }
 
-    bool objectAlreadyExists = false;
     UMLObject   *obj = 0;
-    UMLObjectListIt object_it(objects);
     doc->setModified(true);
     idchanges = doc->getChangeLog();
+    // Assume success if at least one child object could be pasted
+    if (objects.count())
+        result = false;
 
-    while ( (obj=object_it.current()) != 0 ) {
+    for (UMLObjectListIt it(objects); (obj = it.current()) != NULL; ++it) {
         obj->setID(doc->assignNewID(obj->getID()));
         switch(obj->getBaseType()) {
         case Uml::ot_Attribute :
             {
-                if (parent->addAttribute(dynamic_cast<UMLAttribute*>(obj), idchanges)) {
-                    //FIXME                                     doc -> signalChildUMLObjectCreated(obj);
+                UMLObject *exist = parent->findChildObject(obj->getName(), Uml::ot_Attribute);
+                if (exist) {
+                    QString newName = parent->uniqChildName(Uml::ot_Attribute, obj->getName());
+                    obj->setName(newName);
+                }
+                UMLAttribute *att = static_cast<UMLAttribute*>(obj);
+                if (parent->addAttribute(att, idchanges)) {
+                    result = true;
                 } else {
-                    objectAlreadyExists = true;
+                    kError() << "UMLClipboard::pasteClip5: " << parent->getName()
+                        << "->addAttribute(" << att->getName() << ") failed" << endl;
                 }
                 break;
             }
         case Uml::ot_Operation :
             {
-                UMLClassifier * parent = dynamic_cast<UMLClassifier *>(lvitem -> getUMLObject());
-                if (parent -> addOperation(dynamic_cast<UMLOperation*>(obj), idchanges)) {
-                    //FIXME                             doc -> signalChildUMLObjectCreated(obj);
+                UMLOperation *op = static_cast<UMLOperation*>(obj);
+                UMLOperation *exist = parent->checkOperationSignature(op->getName(), op->getParmList());
+                if (exist) {
+                    QString newName = parent->uniqChildName(Uml::ot_Operation, obj->getName());
+                    op->setName(newName);
+                }
+                if (parent->addOperation(op, idchanges)) {
+                    result = true;
                 } else {
-                    objectAlreadyExists = true;
+                    kError() << "UMLClipboard::pasteClip5: " << parent->getName()
+                        << "->addOperation(" << op->getName() << ") failed" << endl;
                 }
                 break;
             }
         default :
-            kWarning() << "pasteing unknown children type in clip type 5" << endl;
+            kWarning() << "pasting unknown children type in clip type 5" << endl;
             return false;
         }
-        ++object_it;
-    }
-
-    if (objectAlreadyExists) {
-        pasteItemAlreadyExists();
     }
 
     return result;
