@@ -5,7 +5,7 @@
     source code is not replicated in the XMI file.
                              -------------------
     copyright            : (C) 2003 Brian Thomas brian.thomas@gsfc.nasa.gov
-      (C) 2004-2006  Umbrello UML Modeller Authors <uml-devel@uml.sf.net> 
+      (C) 2004-2006  Umbrello UML Modeller Authors <uml-devel@uml.sf.net>
 ***************************************************************************/
 
 /***************************************************************************
@@ -31,6 +31,8 @@
 #include "../operation.h"
 #include "../attribute.h"
 #include "../association.h"
+#include "../template.h"
+#include "../umltemplatelist.h"
 
 JavaWriter::JavaWriter() {
     startline = m_endl + m_indentation;
@@ -113,10 +115,12 @@ void JavaWriter::writeClass(UMLClassifier *c)
 
     // another preparation, determine what we have
     UMLAssociationList associations = c->getSpecificAssocs(Uml::at_Association); // BAD! only way to get "general" associations.
+    UMLAssociationList uniAssociations = c->getUniAssociationToBeImplemented();
+
     UMLAssociationList aggregations = c->getAggregations();
     UMLAssociationList compositions = c->getCompositions();
 
-    bool hasAssociations = aggregations.count() > 0 || associations.count() > 0 || compositions.count() > 0;
+    bool hasAssociations = aggregations.count() > 0 || associations.count() > 0 || compositions.count() > 0 || uniAssociations.count() > 0;
     bool hasAttributes = (atl.count() > 0);
     bool hasAccessorMethods = hasAttributes || hasAssociations;
     bool hasOperationMethods = (c->getOpList().count() > 0);
@@ -186,6 +190,7 @@ void JavaWriter::writeClass(UMLClassifier *c)
     writeAttributeDecls(atpub, atprot, atpriv, java);
 
     writeAssociationDecls(associations, c->getID(), java);
+    writeAssociationDecls(uniAssociations, c->getID(), java);
     writeAssociationDecls(aggregations, c->getID(), java);
     writeAssociationDecls(compositions, c->getID(), java);
 
@@ -230,6 +235,7 @@ void JavaWriter::writeClass(UMLClassifier *c)
 
     // first: determine the name of the other class
     writeAssociationMethods(associations, c, java);
+    writeAssociationMethods(uniAssociations, c, java);
     writeAssociationMethods(aggregations, c, java);
     writeAssociationMethods(compositions, c, java);
 
@@ -280,7 +286,7 @@ void JavaWriter::writeClassDecl(UMLClassifier *c, QTextStream &java)
     } else
         scope = "public ";
 
-    java<<(c->getAbstract()?QString("abstract "):QString(""))<<scope;
+    java<<((c->getAbstract() && !isInterface) ? QString("abstract ") : QString(""))<<scope;
     if(isInterface)
         java<<"interface ";
     else
@@ -288,20 +294,60 @@ void JavaWriter::writeClassDecl(UMLClassifier *c, QTextStream &java)
 
     java<<classname;
 
+    // Generics
+    UMLTemplateList template_params = c->getTemplateList();
+    if (template_params.count()) {
+        java << "<";
+        for (UMLTemplate *t = template_params.first(); t; ) {
+            QString formalName = t->getName();
+            java <<  formalName;
+            QString typeName = t->getTypeName();
+            if (typeName != "class") {
+                java << " extends " << typeName;
+            }
+            if ((t = template_params.next()) != NULL)
+                java << ", ";
+        }
+        java << ">" << m_endl;
+    }
 
     // write inheritances out
     UMLClassifier *concept;
-    UMLClassifierList superclasses = c->findSuperClassConcepts();
-
-    if(superclasses.count()>0)
-        java<<" extends ";
-
-    // Q: Where is 'implements' ??
+    UMLClassifierList superclasses = c->findSuperClassConcepts(UMLClassifier::CLASS);
 
     int i = 0;
     for (concept= superclasses.first(); concept; concept = superclasses.next())
     {
-        java<<cleanName(concept->getName())<<(i>0?", ":"");
+        if (i == 0)
+        {
+            java<< " extends ";
+        }
+        else
+        {
+            //The java generated code is wrong ! : No multiple inheritence of class
+            java<< ", " ;
+        }
+        java<< cleanName(concept->getName());
+        i++;
+    }
+
+    UMLClassifierList superInterfaces = c->findSuperClassConcepts(UMLClassifier::INTERFACE);
+    i = 0;
+    for (concept= superInterfaces.first(); concept; concept = superInterfaces.next())
+    {
+        if (i == 0)
+        {
+            if (isInterface)
+                java<< " extends ";
+            else
+                java<< " implements ";
+        }
+        else
+        {
+            //The java generated code is OK ! : multiple inheritence of interface
+            java<< ", " ;
+        }
+        java<< cleanName(concept->getName());
         i++;
     }
 
@@ -485,7 +531,7 @@ void JavaWriter::writeAssociationRoleDecl(QString fieldClassName,
     // multiplicity object that we don't have to figure out what it means via regex.
     if(multi.isEmpty() || multi.contains(QRegExp("^[01]$")))
     {
-        QString fieldVarName = roleName.replace(0, 1, roleName.left(1).lower());
+        QString fieldVarName = "m_" + roleName.replace(0, 1, roleName.left(1).lower());
         java<<startline<<scope<<" "<<fieldClassName<<" "<<fieldVarName<<";";
     }
     else
@@ -544,7 +590,7 @@ void JavaWriter::writeAssociationRoleMethod (QString fieldClassName, QString rol
 {
     if(multi.isEmpty() || multi.contains(QRegExp("^[01]$")))
     {
-        QString fieldVarName = "m_" + roleName.lower();
+        QString fieldVarName = "m_" + roleName.replace(0, 1, roleName.left(1).lower());
         writeSingleAttributeAccessorMethods(fieldClassName, fieldVarName, roleName,
                                             description, visib, change, false, java);
     }
@@ -666,6 +712,76 @@ QStringList JavaWriter::defaultDatatypes() {
     return l;
 }
 
+
+bool JavaWriter::compareJavaMethod(UMLOperation *op1, UMLOperation *op2)
+{
+    if (op1 == NULL || op2 == NULL)
+        return false;
+    if (op1 == op2)
+        return true;
+    if (op1->getName() != op2->getName())
+        return false;
+    UMLAttributeList atl1 = op1->getParmList();
+    UMLAttributeList atl2 = op2->getParmList();
+    if (atl1.count() != atl2.count())
+        return false;
+    UMLAttribute *at1;
+    UMLAttribute *at2;
+    for (at1  = atl1.first(), at2 = atl2.first(); at1 && at2 ; at1 = atl1.next(),at2 = atl2.next())
+    {
+        if (at1->getTypeName() != at2->getTypeName())
+            return false;
+    }
+    return true;
+
+}
+
+bool JavaWriter::javaMethodInList(UMLOperation *umlOp, UMLOperationList &opl)
+{
+    for (UMLOperation *op = opl.first(); op; op = opl.next()) {
+        if (JavaWriter::compareJavaMethod(op, umlOp)) {
+            return true;
+        }
+    }
+    return false;
+}
+
+void JavaWriter::getSuperImplementedOperations(UMLClassifier *c, UMLOperationList &yetImplementedOpList ,UMLOperationList &toBeImplementedOpList, bool noClassInPath)
+{
+    UMLClassifierList superClasses = c->findSuperClassConcepts();
+
+    for (UMLClassifier *concept= superClasses.first(); concept; concept = superClasses.next())
+    {
+        getSuperImplementedOperations(concept, yetImplementedOpList, toBeImplementedOpList, (concept->isInterface() && noClassInPath));
+        UMLOperationList opl = concept->getOpList();
+        for (UMLOperation *op = opl.first(); op; op = opl.next()) {
+            if (concept->isInterface() && noClassInPath) {
+                if (!JavaWriter::javaMethodInList(op,toBeImplementedOpList))
+                    toBeImplementedOpList.append(op);
+            }
+            else
+            {
+                if (!JavaWriter::javaMethodInList(op, yetImplementedOpList))
+                    yetImplementedOpList.append(op);
+            }
+        }
+    }
+
+}
+
+void JavaWriter::getInterfacesOperationsToBeImplemented(UMLClassifier *c, UMLOperationList &opList )
+{
+    UMLOperationList yetImplementedOpList;
+    UMLOperationList toBeImplementedOpList;
+
+    getSuperImplementedOperations(c,yetImplementedOpList, toBeImplementedOpList);
+    for (UMLOperation *op = toBeImplementedOpList.first(); op; op = toBeImplementedOpList.next())
+    {
+        if ( ! JavaWriter::javaMethodInList(op, yetImplementedOpList) && ! JavaWriter::javaMethodInList(op, opList) )
+            opList.append(op);
+    }
+}
+
 void JavaWriter::writeOperations(UMLClassifier *c, QTextStream &java) {
     UMLOperationList opl;
     UMLOperationList oppub,opprot,oppriv;
@@ -673,8 +789,11 @@ void JavaWriter::writeOperations(UMLClassifier *c, QTextStream &java) {
     opprot.setAutoDelete(false);
     oppriv.setAutoDelete(false);
 
-    //sort operations by scope first and see if there are abstrat methods
+    //sort operations by scope first and see if there are abstract methods
     opl = c->getOpList();
+    if (! c->isInterface()) {
+        getInterfacesOperationsToBeImplemented(c, opl);
+    }
     for (UMLOperation *op = opl.first(); op; op = opl.next()) {
         switch(op->getVisibility()) {
           case Uml::Visibility::Public:
@@ -740,7 +859,7 @@ void JavaWriter::writeOperations(UMLOperationList &oplist, QTextStream &java) {
             returnStr += "@return       "+methodReturnType+"\n";
 
         str = ""; // reset for next method
-        str += ((op->getAbstract() || isInterface) ? "abstract ":"");
+        str += ((op->getAbstract() && !isInterface) ? "abstract ":"");
         str += scopeToJavaDecl(op->getVisibility()) + ' ';
         str += (op->getStatic() ? "static ":"");
         str += methodReturnType + ' ' +cleanName(op->getName()) + "( ";
