@@ -6,7 +6,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   copyright (C) 2006                                                    *
- *   Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>                 *
+ *   Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                  *
  ***************************************************************************/
 
 #include "pascalwriter.h"
@@ -25,7 +25,7 @@
 #include "../classifierlistitem.h"
 #include "../umlclassifierlistitemlist.h"
 #include "../umltemplatelist.h"
-#include "../package.h"
+#include "../folder.h"
 #include "../association.h"
 #include "../attribute.h"
 #include "../operation.h"
@@ -55,7 +55,7 @@ bool PascalWriter::isOOClass(UMLClassifier *c) {
     if (ot == Uml::ot_Enum)
         return false;
     if (ot != Uml::ot_Class) {
-        kdDebug() << "PascalWriter::isOOClass: unknown object type " << ot << endl;
+        kDebug() << "PascalWriter::isOOClass: unknown object type " << ot << endl;
         return false;
     }
     QString stype = c->getStereotype();
@@ -67,14 +67,18 @@ bool PascalWriter::isOOClass(UMLClassifier *c) {
     return true;
 }
 
-QString PascalWriter::qualifiedName(UMLClassifier *c, bool withType, bool byValue) {
-    UMLPackage *umlPkg = c->getUMLPackage();
-    QString className = cleanName(c->getName());
+QString PascalWriter::qualifiedName(UMLPackage *p, bool withType, bool byValue) {
+    UMLPackage *umlPkg = p->getUMLPackage();
+    QString className = cleanName(p->getName());
     QString retval;
 
+    if (umlPkg == UMLApp::app()->getDocument()->getRootFolder(Uml::mt_Logical))
+        umlPkg = NULL;
+
+    UMLClassifier *c = dynamic_cast<UMLClassifier*>(p);
     if (umlPkg == NULL) {
         retval = className;
-        if (! isOOClass(c))
+        if (c == NULL || !isOOClass(c))
             retval.append(defaultPackageSuffix);
     } else {
         retval = umlPkg->getFullyQualifiedName(".");
@@ -85,7 +89,7 @@ QString PascalWriter::qualifiedName(UMLClassifier *c, bool withType, bool byValu
     }
     if (! withType)
         return retval;
-    if (isOOClass(c)) {
+    if (c && isOOClass(c)) {
         retval.append(".Object");
         if (! byValue)
             retval.append("_Ptr");
@@ -118,7 +122,7 @@ void PascalWriter::computeAssocTypeAndRole
 
 void PascalWriter::writeClass(UMLClassifier *c) {
     if (!c) {
-        kdDebug() << "Cannot write class of NULL concept!" << endl;
+        kDebug() << "Cannot write class of NULL concept!" << endl;
         return;
     }
 
@@ -156,12 +160,12 @@ void PascalWriter::writeClass(UMLClassifier *c) {
     pas << "unit " << unit << ";" << m_endl << m_endl;
     pas << "INTERFACE" << m_endl << m_endl;
     // Use referenced classes.
-    UMLClassifierList imports;
+    UMLPackageList imports;
     findObjectsRelated(c, imports);
     if (imports.count()) {
         pas << "uses" << m_endl;
         bool first = true;
-        for (UMLClassifier *con = imports.first(); con; con = imports.next()) {
+        for (UMLPackage *con = imports.first(); con; con = imports.next()) {
             if (con->getBaseType() != Uml::ot_Datatype) {
                 if (first)
                     first = false;
@@ -322,7 +326,7 @@ void PascalWriter::writeOperation(UMLOperation *op, QTextStream &pas, bool is_co
         pas << "// TODO: generate status method " << op->getName() << m_endl;
         return;
     }
-    UMLAttributeList *atl = op->getParmList();
+    UMLAttributeList atl = op->getParmList();
     QString rettype = op->getTypeName();
     bool use_procedure = (rettype.isEmpty() || rettype == "void");
 
@@ -334,11 +338,11 @@ void PascalWriter::writeOperation(UMLOperation *op, QTextStream &pas, bool is_co
     else
         pas << "function ";
     pas << cleanName(op->getName()) << " ";
-    if (atl->count()) {
+    if (atl.count()) {
         pas << "(" << m_endl;
         uint i = 0;
         m_indentLevel++;
-        for (UMLAttribute *at = atl->first(); at; at = atl->next()) {
+        for (UMLAttribute *at = atl.first(); at; at = atl.next()) {
             pas << getIndent();
             if (is_comment)
                 pas << "// ";
@@ -349,12 +353,12 @@ void PascalWriter::writeOperation(UMLOperation *op, QTextStream &pas, bool is_co
             pas << at->getTypeName();
             if (! at->getInitialValue().isEmpty())
                 pas << " := " << at->getInitialValue();
-            if (++i < (uint)atl->count())
+            if (++i < (uint)atl.count())
                 pas << ";" << m_endl;
         }
         m_indentLevel--;
+        pas << ")";
     }
-    pas << ")";
     if (! use_procedure)
         pas << " : " << rettype;
     pas << "; virtual; abstract;" << m_endl << m_endl;
