@@ -13,16 +13,18 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2006                                                    *
+ *   copyright (C) 2006-2007                                               *
  *   Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                  *
  ***************************************************************************/
 
-#include <kdebug.h>
-
-#include <qregexp.h>
-
+// own header
 #include "rubycodeaccessormethod.h"
 
+// qt/kde includes
+#include <qregexp.h>
+#include <kdebug.h>
+
+// local includes
 #include "../attribute.h"
 #include "../codegenerator.h"
 #include "../classifiercodedocument.h"
@@ -45,8 +47,8 @@ RubyCodeAccessorMethod::RubyCodeAccessorMethod ( CodeClassField * field, CodeAcc
     setType(type);
 
     // lets use full-blown comment
-    setComment(new RubyCodeDocumentation((RubyClassifierCodeDocument*)field->getParentDocument()));
-
+    RubyClassifierCodeDocument *rccd = dynamic_cast<RubyClassifierCodeDocument*>(field->getParentDocument());
+    setComment(new RubyCodeDocumentation(rccd));
 }
 
 RubyCodeAccessorMethod::~RubyCodeAccessorMethod ( ) { }
@@ -77,7 +79,7 @@ void RubyCodeAccessorMethod::updateContent( )
 {
 
     CodeClassField * parentField = getParentClassField();
-    RubyCodeClassField * rubyfield = (RubyCodeClassField*)parentField;
+    RubyCodeClassField * rubyfield = dynamic_cast<RubyCodeClassField*>(parentField);
     QString fieldName = rubyfield->getFieldName();
     QString endLine = UMLApp::app()->getCommonPolicy()->getNewLineEndingChars();
 
@@ -86,7 +88,6 @@ void RubyCodeAccessorMethod::updateContent( )
     case CodeAccessorMethod::ADD:
         {
             int maxOccurs = rubyfield->maximumListOccurances();
-            RubyClassifierCodeDocument * rubydoc = (RubyClassifierCodeDocument*) rubyfield->getParentDocument();
             QString fieldType = rubyfield->getTypeName();
             QString indent = getIndentation();
             if(maxOccurs > 0)
@@ -108,7 +109,7 @@ void RubyCodeAccessorMethod::updateContent( )
     case CodeAccessorMethod::REMOVE:
         {
             int minOccurs = rubyfield->minimumListOccurances();
-            RubyClassifierCodeDocument * rubydoc = (RubyClassifierCodeDocument*) rubyfield->getParentDocument();
+            RubyClassifierCodeDocument * rubydoc = dynamic_cast<RubyClassifierCodeDocument*>(rubyfield->getParentDocument());
             QString fieldType = rubyfield->getTypeName();
             QString indent = getIndentation();
 
@@ -137,8 +138,8 @@ void RubyCodeAccessorMethod::updateContent( )
 void RubyCodeAccessorMethod::updateMethodDeclaration()
 {
 
-    RubyCodeClassField * rubyfield = (RubyCodeClassField*) getParentClassField();
-    RubyClassifierCodeDocument * rubydoc = (RubyClassifierCodeDocument*) rubyfield->getParentDocument();
+    RubyCodeClassField * rubyfield = dynamic_cast<RubyCodeClassField*>(getParentClassField());
+    RubyClassifierCodeDocument * rubydoc = dynamic_cast<RubyClassifierCodeDocument*>(rubyfield->getParentDocument());
 
     // gather defs
     CodeGenerationPolicy *p = UMLApp::app()->getCommonPolicy();
@@ -150,7 +151,7 @@ void RubyCodeAccessorMethod::updateMethodDeclaration()
     if(objectType.isEmpty())
         objectType = fieldName;
     QString endLine = p->getNewLineEndingChars();
-    
+
     QString description = getParentObject()->getDoc();
     description.replace(QRegExp("m_[npb](?=[A-Z])"), "");
     description.replace("m_", "");
@@ -187,7 +188,7 @@ void RubyCodeAccessorMethod::updateMethodDeclaration()
         setEndMethodText("end");
         break;
     case CodeAccessorMethod::GET:
-        headerText = "Get the value of " + fieldName + endLine + description; 
+        headerText = "Get the value of " + fieldName + endLine + description;
         setStartMethodText(QString("attr_reader :") + fieldName);
         setEndMethodText("");
         break;
