@@ -13,6 +13,8 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   copyright (C) 2006-2007                                               *
+ *   Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                  *
  ***************************************************************************/
 
 #ifndef RUBYCODEGENERATOR_H
@@ -90,7 +92,7 @@ public:
 
     /**
      * @return    ClassifierCodeDocument
-     * @param    classifier 
+     * @param    classifier
      */
     CodeDocument * newClassifierCodeDocument (UMLClassifier * classifier);
 
@@ -98,7 +100,7 @@ public:
     Uml::Programming_Language getLanguage();
 
     /**
-     * Convert a C++ type such as 'int' or 'QWidget' to 
+     * Convert a C++ type such as 'int' or 'QWidget' to
      * ruby types Integer and Qt::Widget
      *
      * @param cppType the C++ type to be converted
@@ -106,7 +108,7 @@ public:
     static QString cppToRubyType(const QString &cppType);
 
     /**
-     * Convert C++ names such as 'm_foobar' or pFoobar to 
+     * Convert C++ names such as 'm_foobar' or pFoobar to
      * just 'foobar' for ruby
      *
      * @param cppName the C++ name to be converted
@@ -125,9 +127,6 @@ protected:
      */
     CodeBlockWithComments * createClassDecl ( UMLClassifier *c, ClassifierInfo *info, RubyClassifierCodeDocument * doc);
 
-private:
-
-    void initFields( ) ;
 };
 
 #endif // RUBYCODEGENERATOR_H
